@@ -4,8 +4,8 @@
 #'
 #' The arguments \code{width}, \code{height}, and \code{res} are passed on to \code{\link[grDevices:png]{png}}. If \code{x} is a tmap object, a temporarily png image is created to calculate the aspect ratio of a tmap object. The default size of this image is 700 by 700 pixels at 100 dpi.
 #'
-#' @param x shape object (either \code{\link[sp:Spatial]{Spatial}}, a \code{\link[raster:Raster-class]{Raster}}, or an \code{sf}), a bounding box (that can be coerced by \code{\link{bb}}), or a \code{\link[tmap:tmap-element]{tmap}} object.
-#' @param is.projected Logical that determined whether the coordinates of \code{x} are projected (\code{TRUE}) or longitude latitude coordinates (\code{FALSE}). By default, it is determined by the coordinates of \code{x}.
+#' @param x shape object (either \code{\link[sp:Spatial]{Spatial}}, a \code{\link[raster:Raster-class]{Raster}}, or an \code{\link[sf:sf]{sf}}), a bounding box (that can be coerced by \code{\link{bb}}), or a \code{\link[tmap:tmap-element]{tmap}} object.
+#' @param is.projected Logical that determined wether the coordinates of \code{x} are projected (\code{TRUE}) or longitude latitude coordinates (\code{FALSE}). By deafult, it is determined by the coordinates of \code{x}.
 #' @param width See details; only applicable if \code{x} is a \code{\link[tmap:tmap-element]{tmap}} object.
 #' @param height See details; only applicable if \code{x} is a \code{\link[tmap:tmap-element]{tmap}} object.
 #' @param res See details; only applicable if \code{x} is a \code{\link[tmap:tmap-element]{tmap}} object.
@@ -13,13 +13,12 @@
 #' @import sp
 #' @importFrom raster couldBeLonLat
 #' @example ./examples/get_asp_ratio.R
-#' @references Tennekes, M., 2018, {tmap}: Thematic Maps in {R}, Journal of Statistical Software, 84(6), 1-39, \href{https://doi.org/10.18637/jss.v084.i06}{DOI}
 #' @export
 get_asp_ratio <- function(x, is.projected = NA, width=700, height=700, res=100) {
 	if (inherits(x, "tmap")) {
 		tmp <- tempfile(fileext = ".png")
 		png(tmp, width=width, height=height, res = res)
-		asp <- print(x, return.asp = TRUE)
+		asp <- print(x, return.asp = TRUE, mode = "plot")
 		dev.off()
 	} else {
 	    if (inherits(x, c("Spatial", "Raster", "sf", "sfc"))) {
@@ -30,8 +29,8 @@ get_asp_ratio <- function(x, is.projected = NA, width=700, height=700, res=100)
 	        if (is.na(is.projected)) is.projected <- !maybe_longlat(bbx)
 	    }
 
-	    xlim <- bbx[1, ]
-	    ylim <- bbx[2, ]
+	    xlim <- bbx[c(1,3)]
+	    ylim <- bbx[c(2,4)]
 
 	    asp <- if (diff(xlim)==0 || diff(ylim)==0) {
 	        1
